//
//  AudioRoom-Player.h
//  zegoaudioroom
//
//  Copyright © 2017年 ZEGO. All rights reserved.
//

#ifndef AudioRoom_Player_h
#define AudioRoom_Player_h

#include "./AudioRoomDefines.h"
#include "./AudioRoomCallback-Player.h"

namespace AVE
{
    struct AudioFrame;
    struct ExtAudioProcSet;
    typedef ExtAudioProcSet ExtPostpSet;

    /**
     音频后处理回调函数定义
     
     @param streamId 流 ID, 当 streamId 为空串时，表明这是混音后的音频数据
     @param inFrame 待处理音频数据
     @param outFrame 处理后的音频数据，其中 buffer 所指向的内存由 SDK 负责申请和销毁，用户处理后回写的数据的采样率不超过48K
     @attention 请确保在当前线程完成，且不要做耗时操作
     */
    typedef void (*OnPostpCallback)(const char* streamId, const AudioFrame& inFrame, AudioFrame& outFrame);
}

namespace ZEGO
{
    namespace AUDIOROOM
    {
		/**
		开始播放流

		@param pszStreamID 播放流流 ID 
		@attention 设置手动播放时有效
		@return true 成功，false 失败
		*/
        ZEGO_API bool StartPlay(const char *pszStreamID);

		/**
		停止播放流

		@param pszStreamID 流 ID
		@attention 设置手动播放时有效
		*/
        ZEGO_API void StopPlay(const char *pszStreamID);

        /**
         获取 SDK 支持的最大同时播放流数
         
         @return 最大支持播放流数
         */
        ZEGO_API int GetMaxPlayChannelCount();
        
        /**
         设置直播观众相关信息通知的回调
         
         @param pCB 回调对象指针
         @return true 成功，false 失败
         */
        ZEGO_API bool SetAudioLivePlayerCallback(IAudioLivePlayerCallback *pCB);
        
        /**
         （声音输出）静音开关
         
         @param bEnable true 不静音，false 静音。默认 true
         @return true 成功，false 失败
         @attention 设置为关闭后，默认扬声器和耳机均无声音输出
         */
        ZEGO_API bool EnableSpeaker(bool bEnable);
       
        /**
         默认扬声器开关
         
         @param bOn true 打开，false 关闭。默认 true
         @return true 成功，false 失败
         @attention 设置为关闭后，扬声器无声音，耳机仍有声音输出
         */
        ZEGO_API bool SetBuiltInSpeakerOn(bool bOn);
        
        /**
         设置播放音量
         
         @param volume 音量大小, 取值范围[0, 200], 默认 100
         @attention 直播时通过此 API 软件调整音量
         */
        ZEGO_API bool SetPlayVolume(int volume);
        
        /**
         获取当前播放视频的音量
         
         @param pszStreamID 播放流 ID
         @return 视频的音量值
         @attention 直播时通过此 API 获取当前音量。
         */
        ZEGO_API float GetSoundLevel(const char* pszStreamID);
        
        /**
         设置音频后处理函数

         @param callback 音频后处理函数指针
         @param config 预处理的采样率等参数设置
         @attention 必须在 InitSDK 后且在拉流前调用
         @attention 仅在使用 EnableAudioPostp 开启了音频后处理特性后才会回调
         @attention 请确保在当前线程完成，且不要做耗时操作
         */
        ZEGO_API void SetAudioPostpCallback(AVE::OnPostpCallback callback, const AVE::ExtPostpSet& config);
        
        /**
         开启/关闭音频后处理特性
         
         @param enable true: 开启拉流音频后处理特性；false: 关闭拉流音频后处理特性
         @param streamID 流 ID, 当 streamID 为空串时，表明需要混音后的音频数据
         @attention streamID不为空串时，必须在拉流后调用，本次拉流有效
         @attention 当拉流音频后处理特性开启时，待渲染音频数据通过 SetAudioPostpCallback 设置的回调函数指针返回
         */
        ZEGO_API void EnableAudioPostp(bool enable, const char* streamID);
        
        /**
         音频录制回调开关
         
         @param uMask 启用音频源选择，参考 ZegoAVAPIAudioRecordMask
         @param nSampleRate 采样率 8000, 16000, 22050, 24000, 32000, 44100, 48000
		 @param nChannels 声道数 1(单声道) 或 2(双声道) 默认1
         @return true 成功，false 失败
         */
        ZEGO_API bool EnableSelectedAudioRecord(unsigned int uMask, int nSampleRate = 44100, int nChannels = 1);
        
        /**
         设置音频录制回调
         
         @param pCB 回调实现实例
         @return true 成功，false 失败
         */
        ZEGO_API bool SetAudioRecordCallback(AV::IZegoAudioRecordCallback* pCB);

        /**
        onPlayStateUpdate返回拉流失败时，可以尝试再次拉流

        @param streamID 重新拉流流ID
        @return true 调用成功，false 调用失败
        */
        ZEGO_API bool RestartPlayStream(const char *streamID);

        /**
         设置拉流播放缓存自适应调整的区间范围（上下限）

         @param minBufferLevel 缓存自适应区间下限, 单位毫秒. 当 minBufferLevel <= 0 表示未设置下限.
         @param maxBufferLevel 缓存自适应区间上限, 单位毫秒. 当 maxBufferLevel <= 0 表示未设置上限. 当设置的上限超过4000时，会取值4000.
         @param streamID 流 ID
         @return true 设置成功, false 设置失败
         @note 调用时机为拉流前或者拉流中, 每次拉流都需要重新设置
         @note 上下限在未设置时, 内部默认区间范围 [0, 4000]
         @note 当设置的上限小于下限时，会自动将上限取值为下限
         */
        ZEGO_API bool SetRecvBufferLevelLimit(int minBufferLevel, int maxBufferLevel, const char* streamID);

    }
}
#endif /* AudioRoom_Player_h */
