//
//  AudioRoom-Publisher.h
//  zegoaudioroom
//
//  Copyright © 2017年 ZEGO. All rights reserved.
//

#ifndef AudioRoom_Publisher_h
#define AudioRoom_Publisher_h

#include "./AudioRoomDefines.h"
#include "./AudioRoomCallback-Publisher.h"

namespace AVE
{
    struct AudioFrame;
    struct ExtAudioProcSet;
    typedef ExtAudioProcSet ExtPrepSet;

    /**
     音频处理回调函数定义

     @param inFrame 待处理音频数据
     @param outFrame 处理后的音频数据，其中buffer 所指向的内存由 SDK 负责申请和销毁，用户处理后回写的数据的采样率不超过48K
     @attention 请确保在当前线程完成，且不要做耗时操作
     */
    typedef void (*OnAudioProcCallback)(const AVE::AudioFrame& inFrame, AVE::AudioFrame& outFrame);

    typedef OnAudioProcCallback OnPrepCallback;
}

namespace ZEGO
{
    namespace AUDIOROOM
    {
        /**
         设置直播主播相关信息通知的回调
         
         @param pCB 回调对象指针
         @return true 成功，false 失败
         */
        
        ZEGO_API bool SetAudioLivePublisherCallback(IAudioLivePublisherCallback *pCB);
        
        ZEGO_API bool StartPublish();
        
        ZEGO_API bool StartPublish(const char *pszStreamID);
        
        ZEGO_API void StopPublish();


		/**
		启动本地预览

		*/
		ZEGO_API bool StartAudioCapture();

		/**
		结束本地预览

		*/
		ZEGO_API bool StopAudioCapture();
        
		/**
		设置或更新推流的附加信息

		@param pszStreamExtraInfo 流附加信息, 最大为 1024 字节
		@return 更新流附加信息成功后，同一房间内的其他人会收到 OnStreamExtraInfoUpdated 通知
		*/
		ZEGO_API bool SetPublishStreamExtraInfo(const char *pszStreamExtraInfo);

        /**
         开启麦克风
         
         @param bEnable true 打开，false 关闭。默认 false
         @return true 成功，false 失败
         @attention 推流时可调用本 API 进行参数配置
         */
        ZEGO_API bool EnableMic(bool bEnable);
        
        /**
         设置音频码率
         
         @param bitrate 码率
         @return true 成功，false 失败
         */
        ZEGO_API bool SetAudioBitrate(int bitrate);
        
        /**
         设置音频设备模式
         
         @param mode 模式，默认 ZEGO_AUDIO_DEVICE_MODE_AUTO
         @attention 在推流预览前后调用均有效，调用该接口会触发设备的启动切换，建议不要频繁调用，避免不必要的开销与硬件问题
         @attention 用本接口可能导致音量模式在 通话/媒体 间切换，若媒体音量和通话音量不一致，可能导致音量变化
         */
        ZEGO_API void SetAudioDeviceMode(AV::ZegoAVAPIAudioDeviceMode mode);
        
        /**
         音频采集自动增益开关。windows默认开启；android/ios默认关闭；mac默认音频设备模式为ZEGO_AUDIO_DEVICE_MODE_COMMUNICATION或ZEGO_AUDIO_DEVICE_MODE_COMMUNICATION2时开启，否则关闭
         
         @param bEnable true 开启，false 关闭
         @return true 成功，false 失败
         */
        ZEGO_API bool EnableAGC(bool bEnable);
        
        /**
         回声消除开关
         
         @param bEnable true 开启，false 关闭
         @return true 成功，false 失败
         */
        ZEGO_API bool EnableAEC(bool bEnable);
        
        /**
         开启采集监听
         
         @param bEnable true 打开，false 关闭。默认 false
         @return true 成功，false 失败
         @attention 推流时可调用本 API 进行参数配置。连接耳麦时设置才实际生效。开启采集监听，主播方讲话后，会听到自己的声音。
         */
        ZEGO_API bool EnableLoopback(bool bEnable);
        
        /**
         设置监听音量
         
         @param volume 音量大小，取值范围 [0, 200], 默认 60
         @attention 推流时可调用本 API 进行参数配置
         */
        ZEGO_API void SetLoopbackVolume(int volume);
        
        /**
         设置采集音量
         
         @param volume 音量大小，取值范围 [0, 200], 默认 100
         @attention InitSDK成功之后调用
         */
        ZEGO_API void SetCaptureVolume(int volume);
        
        /**
         获取当前采集的音量
         
         @return 当前采集音量大小
         */
        ZEGO_API float GetCaptureSoundLevel();

        /**
         设置音频前(3A处理后、耳返前)处理函数, 并开启/关闭音频前处理特性

         @param callback 音频前处理函数指针
         @param config 预处理的采样率等参数设置
         @attention 必须在 InitSDK 后且在推流前调用
         @attention 使用该回调处理音频影响耳返效果
         */
        ZEGO_API void SetAudioPrepCallback(AVE::OnPrepCallback callback, const AVE::ExtPrepSet& config);

        /**
         设置音频(耳返后)处理函数, 并开启/关闭音频处理特性

         @param callback 音频处理函数指针
         @param config 预处理的采样率等参数设置
         @attention 必须在 InitSDK 后且在推流前调用
         @attention 使用该回调处理音频不影响耳返效果
         */
        ZEGO_API void SetAudioPrepAfterLoopbackCallback(AVE::OnAudioProcCallback callback, const AVE::ExtPrepSet& config);
        
        /**
         设置延迟模式
         
         @param mode 延迟模式，默认 ZEGO_LATENCY_MODE_NORMAL
         @attention 确保在InitSDK之后，推流前调用
         */
        ZEGO_API void SetLatencyMode(AV::ZegoAVAPILatencyMode mode);
        
        /**
         设置推流音频声道数

        @param count 声道数，1 或 2，默认为 1（单声道）
        @attention 必须在InitSDK之后，推流前设置
        @note SetLatencyMode 设置为 ZEGO_LATENCY_MODE_NORMAL, ZEGO_LATENCY_MODE_NORMAL2, ZEGO_LATENCY_MODE_LOW3 才能设置双声道
        @note 在移动端双声道通常需要配合音频前处理才能体现效果。
        */
        ZEGO_API void SetAudioChannelCount(int count);
        
        /**
         是否开启离散音频包发送

         @param bEnable true 开启，推流中检测为静音的数据包不发送；false 关闭，推流中的静音数据包照常发送。默认是 false。
         @attention 推流前调用有效。以 RTC 方式推流，此接口才会生效。
         @attention SDK 默认关闭该特性。
         @note 可以和 EnableVAD 配合使用。
         @note 开启离散音频包发送可以在关闭麦克风或者静音的状态下，停止音频包的发送，节省用户流量。
         @note 如果为音频单流(非混流)转推 CDN，开启此选项将导致 CDN 在设定时间内未收到数据包认为流关闭而断开连接，所以此种情况不建议开启。
         */
        ZEGO_API void EnableDTX(bool bEnable);

        /**
         是否开启语音活动检测

         @param enable true 开启; false 关闭, 默认关闭
         @attention 确保在推流前调用，只有纯 UDP 方案才可以调用此接口
         */
        ZEGO_API void EnableVAD(bool bEnable);

        /**
        OnPublishStateUpdate回调推流失败时，可以调用此函数再次进行推流
        @return true 调用推流成功，false 调用失败
        */
        ZEGO_API bool RestartPublishStream();
        
        
        /**
         设置是否开启音频流量控制

         @param bEnable true 开启; false 关闭
         */
        ZEGO_API void EnableAudioTrafficControl(bool bEnable);

        /**
        音频采集噪声抑制开关

        @param bEnalbe true 开启，false 关闭
        @return true 调用成功，false 调用失败
        */
       ZEGO_API bool EnableNoiseSuppress(bool bEnable);

       /**
         设置音频采集降噪等级
         @param mode 降噪等级，详见 ZegoANSMode 定义
         @return true 成功，false 失败
         @note 仅在 EnableNoiseSuppress 为 true 时有效, 默认为 MEDIUM
        */
       ZEGO_API bool SetNoiseSuppressMode(AV::ZegoANSMode mode);
    }
}
#endif /* AudioRoom_Publisher_h */
