
#ifndef zego_api_auto_mix_stream_defines_h
#define zego_api_auto_mix_stream_defines_h

#include "zego-api-defines.h"
#include <cstddef>

namespace ZEGO
{
namespace AUTOMIXSTREAM
{
    struct ZegoAutoMixStreamOutputTarget
    {
        /** 输出是否为混流地址, true:输出是混流地址, false:输出是流名 */
        bool bIsUrl;
        /** 输出目标 */
        char szTarget[ZEGO_MAX_COMMON_LEN];
        
        ZegoAutoMixStreamOutputTarget()
        {
            bIsUrl = false;
            szTarget[0] = '\0';
        }
    };
    
    /**
     混流输出目的地
     */
    struct ZegoAutoMixStreamOutput
    {
		/** 输出目标列表 */
        ZegoAutoMixStreamOutputTarget* pTargetList;
        
        /** 输出目标个数 */
        int nTargetCount;

        /*
         视频码率
         */
        int nBitrate;
        
        /*
         视频帧率
         */
        int nFps;
        
        /*
         画布 高，视频输出分辨率高
         */
        int nVideoOutHeight;
        
        /*
         画布宽，视频输出分辨率款
         */
        int nVideoOutWidth;
        
        /*
         音频通道数 0单身道，1单身道 2双声道
         */
        int nAudioChannelCnt;
        
        /*
         音频编码 0：HE_AAC  1 AAC_LC 2 MP3
         */
        int nAudioEncID;
        
        /*
         音频码率
         */
        int nAudioBitrate;
        
        /*
         编码模式 0恒定码率，1恒定质量
         */
        int nEncodeMode;
        
        /*
         视频质量参数, 有效值范围 0 到 51, 0最好，51最差, 默认23
         */
        int nEncodeQua;

		ZegoAutoMixStreamOutput()
		{
            pTargetList = NULL;
            nTargetCount = 0;
            nEncodeMode = 0;
			nEncodeQua = 23;
			nBitrate = 0;
			nFps = 0;
			nVideoOutHeight = 0;
			nVideoOutWidth = 0;
			nAudioChannelCnt = 0;
			nAudioEncID = 0;
			nAudioBitrate = 0;
		}
    };
    
    /**
     混流水印信息
     */
    struct ZegoAutoMixStreamWatermark
    {
        /**
         * 水印图片，只支持png、jpg格式。
         * 此值由 ZEGO 提供，开发者先将图片提供给 ZEGO，ZEGO 设置后再反馈水印图片的设置参数。
         */
        char image[ZEGO_MAX_COMMON_LEN];
        
        struct
        {
			/*
			层次
			*/
			int layer;
            /**
             混流画布左上角坐标的第二个值，左上角坐标为 (left, top)
             */
            int top;
            /**
             混流画布左上角坐标的第一个值，即左上角坐标为 (left, top)
             */
            int left;
            /**
             混流画布右下角坐标的第二个值，右下角坐标为 (right, bottom)
             */
            int bottom;
            /**
             混流画布右下角坐标的第一个值，右下角坐标为 (right, bottom)
             */
            int right;
        } layout;
        /**
         *  原点在左上角，top/bottom/left/right 定义如下：
         *
         *  (left, top)-----------------------
         *  |                                |
         *  |                                |
         *  |                                |
         *  |                                |
         *  -------------------(right, bottom)
         */

        ZegoAutoMixStreamWatermark()
        {
            image[0] = '\0';
            layout.layer = 0;
            layout.top = 0;
            layout.left = 0;
            layout.bottom = 0;
            layout.right = 0;
        }
    };
    
    
    /**
     混流配置
     */
    struct ZegoAutoMixStreamConfig
    {
		/**
		* 混流背景颜色，前三个字节为 RGB，即 0xRRGGBBxx。

		* 例如：选取RGB为 #87CEFA 作为背景色，此处写为 0x87CEFA00。
		*/
		int nOutputBackgroundColor;
		/**
		* 混流背景图，支持预设图片，如 (preset-id://xxx)

		* 此值由 ZEGO 提供，开发者先将背景图提供给 ZEGO，ZEGO 设置后再反馈背景图片的设置参数。
		*/
		const char* pOutputBackgroundImage;
		/**
		是否开启音浪。0 关闭, 1 开启. 默认值为 0.
		*/
		int nWithSoundLevel;

		/**
		用户自定义数据
		@note 注意：pUserData自定义的数据通过媒体次要信息的 onRecvMediaSideInfo 接口回调出来。
		*/
		const unsigned char *pUserData;
		/**
		用户自定义数据的长度
		*/
		int nLenOfUserData;

		/*
		 是否启用单流透传开关 默认不启用（1 启用 0 不启用）
		*/
		int  nByPass;

        /*
         是否开启网络时间对齐功能，默认为false
         需要网络校准的流要在推流前调用EnablePublishStreamAlignment开启推流校准网络时间。
        */
        bool bStreamAlignment;

		/*
		   额外参数, 如果有多个, 通过分号隔开(具体联系即构技术支持 比如 video_encode/sei_mode 可参见数据示例)
		*/
		const char* pszExtraParams;

		/*
	       混流请求来源
		*/

		const char* pszMixSource;
        
        /**
         混流输出流列表
         */
        ZegoAutoMixStreamOutput *pOutput;
        
        /**
         混流水印
         */
        ZegoAutoMixStreamWatermark *pOutputWatermark;   

        ZegoAutoMixStreamConfig ()
        : pOutput(NULL)
        , pUserData(0)
        , nLenOfUserData(0)
        , nOutputBackgroundColor(0)
        , pOutputBackgroundImage(0)
        , nWithSoundLevel(0)
        , pOutputWatermark(NULL)
		, pszMixSource(0)
		, pszExtraParams(NULL)
        , nByPass(0)
        , bStreamAlignment(false)
        {
            
        }
    };
    
    struct ZegoAutoMixStreamSoundLevel
    {
        /**
         音浪ID，用于标识流.
         自动混流时, 对应于房间流信息中的 streamNID 字段.
         */
        unsigned int soundLevelID;
        
        /**
         音量level
         */
        unsigned char soundLevel;
        
        /**
         流 ID
         */
        char streamID[ZEGO_MAX_COMMON_LEN];
        
        /**
         流发布者用户 ID
         */
        char userID[ZEGO_MAX_USERID_LEN];
        
        ZegoAutoMixStreamSoundLevel()
        {
            soundLevelID = 0;
            soundLevel = 0;
            streamID[0] = '\0';
            userID[0] = '\0';
        }           
    };

}
}

#endif 
